<?php
require_once 'config.php';

/**
 * User Authentication Functions
 */

function registerUser($userData) {
    global $pdo;
    
    try {
        $sql = "INSERT INTO users (name, email, password, role, created_at) 
                VALUES (:name, :email, :password, :role, NOW())";
        
        $stmt = $pdo->prepare($sql);
        
        $hashedPassword = password_hash($userData['password'], PASSWORD_DEFAULT);
        
        return $stmt->execute([
            'name' => $userData['name'],
            'email' => $userData['email'],
            'password' => $hashedPassword,
            'role' => $userData['role']
        ]);
    } catch(PDOException $e) {
        error_log("Registration Error: " . $e->getMessage());
        return false;
    }
}

function loginUser($email, $password) {
    global $pdo;
    
    try {
        $stmt = $pdo->prepare("SELECT * FROM users WHERE email = :email");
        $stmt->execute(['email' => $email]);
        $user = $stmt->fetch();
        
        if($user && password_verify($password, $user['password'])) {
            $_SESSION['user_id'] = $user['id'];
            $_SESSION['user_role'] = $user['role'];
            $_SESSION['user_name'] = $user['name'];
            return true;
        }
        return false;
    } catch(PDOException $e) {
        error_log("Login Error: " . $e->getMessage());
        return false;
    }
}

function isLoggedIn() {
    return isset($_SESSION['user_id']);
}

function isAdmin() {
    return isset($_SESSION['user_role']) && $_SESSION['user_role'] === 'admin';
}

function logout() {
    session_destroy();
    session_start();
}

/**
 * Appointment Management Functions
 */

function createAppointment($data) {
    global $pdo;
    
    try {
        $sql = "INSERT INTO appointments (patient_id, doctor_id, appointment_date, 
                appointment_time, reason, status, created_at) 
                VALUES (:patient_id, :doctor_id, :date, :time, :reason, :status, NOW())";
        
        $stmt = $pdo->prepare($sql);
        return $stmt->execute([
            'patient_id' => $data['patient_id'],
            'doctor_id' => $data['doctor_id'],
            'date' => $data['date'],
            'time' => $data['time'],
            'reason' => $data['reason'],
            'status' => 'pending'
        ]);
    } catch(PDOException $e) {
        error_log("Appointment Creation Error: " . $e->getMessage());
        return false;
    }
}

function getAppointments($userId, $role) {
    global $pdo;
    
    try {
        $sql = "SELECT a.*, p.name as patient_name, d.name as doctor_name 
                FROM appointments a 
                JOIN users p ON a.patient_id = p.id 
                JOIN users d ON a.doctor_id = d.id 
                WHERE " . ($role === 'doctor' ? "doctor_id" : "patient_id") . " = :user_id
                ORDER BY appointment_date DESC";
        
        $stmt = $pdo->prepare($sql);
        $stmt->execute(['user_id' => $userId]);
        return $stmt->fetchAll();
    } catch(PDOException $e) {
        error_log("Get Appointments Error: " . $e->getMessage());
        return [];
    }
}

/**
 * File Management Functions
 */

function uploadFile($file, $userId) {
    try {
        $fileExtension = strtolower(pathinfo($file['name'], PATHINFO_EXTENSION));
        
        if(!in_array($fileExtension, ALLOWED_FILE_TYPES)) {
            throw new Exception("File type not allowed");
        }
        
        if($file['size'] > MAX_FILE_SIZE) {
            throw new Exception("File size exceeds limit");
        }
        
        $newFileName = uniqid() . '.' . $fileExtension;
        $uploadPath = UPLOAD_PATH . $newFileName;
        
        if(move_uploaded_file($file['tmp_name'], $uploadPath)) {
            return saveFileToDatabase($newFileName, $userId);
        }
        return false;
    } catch(Exception $e) {
        error_log("File Upload Error: " . $e->getMessage());
        return false;
    }
}

function saveFileToDatabase($fileName, $userId) {
    global $pdo;
    
    try {
        $sql = "INSERT INTO files (user_id, file_name, uploaded_at) 
                VALUES (:user_id, :file_name, NOW())";
        
        $stmt = $pdo->prepare($sql);
        return $stmt->execute([
            'user_id' => $userId,
            'file_name' => $fileName
        ]);
    } catch(PDOException $e) {
        error_log("File Database Save Error: " . $e->getMessage());
        return false;
    }
}

/**
 * Utility Functions
 */

function sanitizeInput($data) {
    return htmlspecialchars(strip_tags(trim($data)));
}

function generateRandomString($length = 10) {
    return bin2hex(random_bytes($length));
}

function flashMessage($message, $type = 'info') {
    $_SESSION['flash'] = [
        'message' => $message,
        'type' => $type
    ];
}

function getFlashMessage() {
    if(isset($_SESSION['flash'])) {
        $flash = $_SESSION['flash'];
        unset($_SESSION['flash']);
        return $flash;
    }
    return null;
}

/**
 * Database Schema Functions
 */

function createDatabaseTables() {
    global $pdo;
    
    try {
        // Users Table
        $pdo->exec("CREATE TABLE IF NOT EXISTS users (
            id INT AUTO_INCREMENT PRIMARY KEY,
            name VARCHAR(100) NOT NULL,
            email VARCHAR(100) UNIQUE NOT NULL,
            password VARCHAR(255) NOT NULL,
            role ENUM('patient', 'doctor', 'admin') NOT NULL,
            created_at DATETIME NOT NULL,
            updated_at DATETIME DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP
        )");

        // Appointments Table
        $pdo->exec("CREATE TABLE IF NOT EXISTS appointments (
            id INT AUTO_INCREMENT PRIMARY KEY,
            patient_id INT NOT NULL,
            doctor_id INT NOT NULL,
            appointment_date DATE NOT NULL,
            appointment_time TIME NOT NULL,
            reason TEXT,
            status ENUM('pending', 'confirmed', 'cancelled') NOT NULL,
            created_at DATETIME NOT NULL,
            updated_at DATETIME DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
            FOREIGN KEY (patient_id) REFERENCES users(id),
            FOREIGN KEY (doctor_id) REFERENCES users(id)
        )");

        // Files Table
        $pdo->exec("CREATE TABLE IF NOT EXISTS files (
            id INT AUTO_INCREMENT PRIMARY KEY,
            user_id INT NOT NULL,
            file_name VARCHAR(255) NOT NULL,
            uploaded_at DATETIME NOT NULL,
            FOREIGN KEY (user_id) REFERENCES users(id)
        )");

        return true;
    } catch(PDOException $e) {
        error_log("Database Schema Creation Error: " . $e->getMessage());
        return false;
    }
}